/**
 * @file
 * VuoList_VuoPoint3d C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoPoint3d_H
#define VuoList_VuoPoint3d_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoPoint3d VuoList_VuoPoint3d
 * A list of @ref VuoPoint3d elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoPoint3d_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoPoint3d elements.
 */
typedef const struct VuoList_VuoPoint3d_struct { void *l; } * VuoList_VuoPoint3d;
/// @}
#endif

/**
 * Creates a new list of @ref VuoPoint3d elements.
 */
VuoList_VuoPoint3d VuoListCreate_VuoPoint3d(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoPoint3d to quickly initialize a large list.
 */
VuoList_VuoPoint3d VuoListCreateWithCount_VuoPoint3d(const unsigned long count, const VuoPoint3d value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoPoint3d VuoListCopy_VuoPoint3d(const VuoList_VuoPoint3d list);

/**
 * Returns the @ref VuoPoint3d at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoPoint3d VuoListGetValue_VuoPoint3d(const VuoList_VuoPoint3d list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoPoint3d *VuoListGetData_VuoPoint3d(const VuoList_VuoPoint3d list);

/**
 * Changes the @ref VuoPoint3d at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoPoint3d(const VuoList_VuoPoint3d list, const VuoPoint3d value, const unsigned long index);

/**
 * Inserts the `VuoPoint3d` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoPoint3d(const VuoList_VuoPoint3d list, const VuoPoint3d value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoPoint3d(VuoList_VuoPoint3d list, const VuoPoint3d value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoPoint3d(VuoList_VuoPoint3d list, const VuoPoint3d value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoPoint3d(VuoList_VuoPoint3d list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoPoint3d(VuoList_VuoPoint3d list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoPoint3d(VuoList_VuoPoint3d list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoPoint3d(VuoList_VuoPoint3d list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoPoint3d(VuoList_VuoPoint3d list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoPoint3d(VuoList_VuoPoint3d list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoPoint3d(VuoList_VuoPoint3d list);

/**
 * Removes the `VuoPoint3d` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoPoint3d(VuoList_VuoPoint3d list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoPoint3d(const VuoList_VuoPoint3d list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoPoint3d VuoList_VuoPoint3d_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoPoint3d_getJson(const VuoList_VuoPoint3d value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoPoint3d_getSummary(const VuoList_VuoPoint3d value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoPoint3d VuoList_VuoPoint3d_makeFromString(const char *str);
char * VuoList_VuoPoint3d_getString(const VuoList_VuoPoint3d value);
/// @}

/**
 * @}
 */

#endif
