/**
 * @file
 * VuoList_VuoVertexAttribute C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoVertexAttribute_H
#define VuoList_VuoVertexAttribute_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoVertexAttribute VuoList_VuoVertexAttribute
 * A list of @ref VuoVertexAttribute elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoVertexAttribute_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoVertexAttribute elements.
 */
typedef const struct VuoList_VuoVertexAttribute_struct { void *l; } * VuoList_VuoVertexAttribute;
/// @}
#endif

/**
 * Creates a new list of @ref VuoVertexAttribute elements.
 */
VuoList_VuoVertexAttribute VuoListCreate_VuoVertexAttribute(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoVertexAttribute to quickly initialize a large list.
 */
VuoList_VuoVertexAttribute VuoListCreateWithCount_VuoVertexAttribute(const unsigned long count, const VuoVertexAttribute value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoVertexAttribute VuoListCopy_VuoVertexAttribute(const VuoList_VuoVertexAttribute list);

/**
 * Returns the @ref VuoVertexAttribute at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoVertexAttribute VuoListGetValue_VuoVertexAttribute(const VuoList_VuoVertexAttribute list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoVertexAttribute *VuoListGetData_VuoVertexAttribute(const VuoList_VuoVertexAttribute list);

/**
 * Changes the @ref VuoVertexAttribute at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoVertexAttribute(const VuoList_VuoVertexAttribute list, const VuoVertexAttribute value, const unsigned long index);

/**
 * Inserts the `VuoVertexAttribute` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoVertexAttribute(const VuoList_VuoVertexAttribute list, const VuoVertexAttribute value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoVertexAttribute(VuoList_VuoVertexAttribute list, const VuoVertexAttribute value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoVertexAttribute(VuoList_VuoVertexAttribute list, const VuoVertexAttribute value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoVertexAttribute(VuoList_VuoVertexAttribute list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoVertexAttribute(VuoList_VuoVertexAttribute list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoVertexAttribute(VuoList_VuoVertexAttribute list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoVertexAttribute(VuoList_VuoVertexAttribute list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoVertexAttribute(VuoList_VuoVertexAttribute list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoVertexAttribute(VuoList_VuoVertexAttribute list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoVertexAttribute(VuoList_VuoVertexAttribute list);

/**
 * Removes the `VuoVertexAttribute` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoVertexAttribute(VuoList_VuoVertexAttribute list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoVertexAttribute(const VuoList_VuoVertexAttribute list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoVertexAttribute VuoList_VuoVertexAttribute_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoVertexAttribute_getJson(const VuoList_VuoVertexAttribute value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoVertexAttribute_getSummary(const VuoList_VuoVertexAttribute value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoVertexAttribute VuoList_VuoVertexAttribute_makeFromString(const char *str);
char * VuoList_VuoVertexAttribute_getString(const VuoList_VuoVertexAttribute value);
/// @}

/**
 * @}
 */

#endif
