#pragma once

/**
 * @file
 * VuoMovie interface.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */
 
#ifdef __cplusplus
extern "C"
{
#include "VuoImage.h"
#include "VuoAudioSamples.h"
#include "VuoList_VuoAudioSamples.h"
#include "VuoVideoFrame.h"
#include "VuoText.h"
#include "VuoUrl.h"
#endif

typedef void* VuoVideo;	///< An opaque object for decoding video.

VuoVideo VuoVideo_make(VuoUrl path);

/**
 * Get the next video frame in the queue.  Returns the frame with image and timestamp (in seconds), and the 
 * duration that this frame should be displayed (also in seconds).
 */
bool VuoVideo_nextVideoFrame(VuoVideo player, VuoVideoFrame* frame);

/**
 * Get the next audio bucket in the queue.  The timestamp in seconds is stored in `timestamp`.
 */
bool VuoVideo_nextAudioFrame(VuoVideo player, VuoList_VuoAudioSamples audio, double* timestamp);

/**
 * Return the full path to the currently playing video.
 */
const char* VuoVideo_getPath(VuoVideo player);

/**
 * Set the playback rate for the decoder.  If the value is not 1, audio is discarded.
 */
void VuoVideo_setPlaybackRate(VuoVideo player, VuoReal rate);

/**
 * Return the timestamp of the last decoded video frame.  If no frames have been decoded yet, the timestamp of the first frame is returned.
 */
double VuoVideo_getLastDecodedVideoTimestamp(VuoVideo player);

/**
 * Get the total duration of this movie in seconds.
 */
double VuoVideo_getDuration(VuoVideo player);

/**
 * Seek the decoder to the second.
 */
bool VuoVideo_seekToSecond(VuoVideo player, VuoReal second);

/**
 * True if the video contains audio and there is a decoder capable of reading it.
 */
bool VuoVideo_containsAudio(VuoVideo player);

#ifdef __cplusplus
}
#endif
